/*
 * Copyright (C) 1995-2002 Neil Peter Charley.
 * Portions Copyright (C) 1996 James William Hawtin.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * ident.h
 *
 * Header file for ident server code.
 *
 */

/* $Revision: 1.3 $ */

#if !defined(IDENTH)
 #define IDENTH

#include <time.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>

/* *IF* you get compile errors about not having 'FIONBIO' comment out
 * the next line (you may start getting other problems though
 */
#define NONBLOCKING

#if defined(__sun__)
/* SunOS of some description */
 #if defined(__svr4__)
 /* Solaris, i.e. SunOS 5.x */
  #if !defined(SOLARIS)
   #define SOLARIS
  #endif /* !SOLARIS */
  #define NOALARM
  /* Signal configuration */
  #define USE_SIGACTION
  #define USE_SIGEMPTYSET
  #define HAVE_SA_SIGACTION
  #define SIGTYPE int sigtype
 #else
 /* __sun__ && !__svr4__ (SunOS 4.x presumeably) */
  #define BSD
  #define SIGTYPE int sigtype, int code,  struct sigcontext *scp, char *addr
  #define HAVE_BCOPY
  #define HAVE_BZERO
 #endif /* __svr4__ */
#endif /* __sun__ */

#if defined(__linux__)
 #define HAVE_UNISTDH
 #define SIGTYPE int it 
 #define USE_SIGACTION 
 #if defined(__GLIBC__) && (__GLIBC__ >= 2)
  #define USE_SIGEMPTYSET
 #endif /* GLIBC >= 2 */
#endif /* __linux__ */

#if defined(__FreeBSD__) || defined(__NetBSD__)
 #define SIGTYPE int i
 #define USE_SIGACTION
 #define USE_SIGEMPTYSET
 #define HAVE_UNISTDH
 #define HAVE_BZERO
#endif /* __FreeBSD__ || __NetBSD__ */

/* Toggle debug mode */
#if defined(RESORT)
 #define DEBUG_IDENT
#endif /* RESORT */

/* The various fd's for the two pipes between client and server */

#define IDENT_SERVER_READ ident_toserver_fds[0]
#define IDENT_SERVER_WRITE ident_toclient_fds[1]
#define IDENT_CLIENT_READ ident_toclient_fds[0]
#define IDENT_CLIENT_WRITE ident_toserver_fds[1]

/* Messages between client and server */

#define IDENT_SERVER_CONNECT_MSG "Server Connect:"
#define IDENT_CLIENT_CONNECT_MSG "Client Connect:"
#define IDENT_CLIENT_SEND_REQUEST 'A'
#define IDENT_CLIENT_CANCEL_REQUEST 'B'
#define IDENT_SERVER_SEND_REPLY ':'

/* Misc #define's */

#ifdef RESORT
 #define MAX_IDENTS_IN_PROGRESS 70
#else
 #define MAX_IDENTS_IN_PROGRESS 20
#endif
#define IDENT_TIMEOUT 60
#define IDENT_NOTYET "<NOT-YET>"
#define IDENT_PORT 113
#define AWAITING_IDENT -1

/* Typedef's */

typedef int ident_identifier;

/* Struct for an ident request */

struct s_ident_request {
	ident_identifier ident_id;
	struct sockaddr_in sock_addr;
	struct sockaddr_in local_sockaddr;
	short int local_port;
	int fd;
	int flags;
	time_t request_time;
	struct s_ident_request *next;
};
typedef struct s_ident_request ident_request;

/* Flags on an ident request */

#define IDENT_CONNECTED   (1 << 0)   /* Connection established */
#define IDENT_WRITTEN     (1 << 1)   /* Request written */
#define IDENT_REPLY_READY (1 << 2)   /* Reply available */
#define IDENT_CONNREFUSED (1 << 3)   /* Connection was refused */
#define IDENT_TIMEDOUT    (1 << 4)   /* Timed out before reply */

#endif /* IDENT_SERVERH */
